using System;
using System.Linq;

namespace ET
{
    [FriendClass(typeof (ServerInfosComponent))]
    [FriendClass(typeof (RoleInfosComponent))]
    public static class RobotManagerComponentSystem
    {
        public static async ETTask<Scene> NewRobot(this RobotManagerComponent self, int zone)
        {
            Scene zoneScene = null;
            try
            {
                zoneScene = SceneFactory.CreateZoneScene(zone, "Robot", self);
                // await LoginHelper.Login(zoneScene, ConstValue.LoginAddress, "robot"+zone.ToString(), zone.ToString());
                //  await EnterMapHelper.EnterMapAsync(zoneScene);
                await LoginRobot(zoneScene, ConstValue.LoginAddress, zone);
                Log.Debug($"create robot ok: {zone}");
                return zoneScene;
            }
            catch (Exception e)
            {
                zoneScene?.Dispose();
                throw new Exception($"RobotSceneManagerComponent create robot fail, zone: {zone}", e);
            }
        }

        public static void RemoveAll(this RobotManagerComponent self)
        {
            foreach (Entity robot in self.Children.Values.ToArray())
            {
                robot.Dispose();
            }
        }

        public static void Remove(this RobotManagerComponent self, long id)
        {
            self.GetChild<Scene>(id)?.Dispose();
        }

        public static void Clear(this RobotManagerComponent self)
        {
            foreach (Entity entity in self.Children.Values.ToArray())
            {
                entity.Dispose();
            }
        }

        //add by wds 机器人登陆
        public static async ETTask<int> LoginRobot(Scene zoneScene, string address, int zone)
        {
            //登陆gate
            int result = await LoginHelper.Login(zoneScene, ConstValue.LoginAddress, "robot" + zone.ToString(), zone.ToString());
            if (result == ErrorCode.ERR_Success)
            {
                result = await LoginHelper.GetServerInfos(zoneScene);
                if (result == ErrorCode.ERR_Success)
                {
                    zoneScene.GetComponent<ServerInfosComponent>().CurSelectedServerId = 1;
                    result = await LoginHelper.GetRoles(zoneScene);
                    if (result == ErrorCode.ERR_Success)
                    {
                        if (zoneScene.GetComponent<RoleInfosComponent>().RoleInfos.Count == 0)
                        {
                            result = await LoginHelper.CreateRole(zoneScene, "robot" + zone.ToString());
                            if (result == ErrorCode.ERR_Success)
                            {
                                zoneScene.GetComponent<RoleInfosComponent>().CurRoleId = zoneScene.GetComponent<RoleInfosComponent>().RoleInfos[0].Id;
                                result = await LoginHelper.GetRealmKey(zoneScene);
                                if (result == ErrorCode.ERR_Success)
                                {
                                    await LoginHelper.EnterGame(zoneScene);
                                }
                            }
                        }
                        else
                        {
                            zoneScene.GetComponent<RoleInfosComponent>().CurRoleId = zoneScene.GetComponent<RoleInfosComponent>().RoleInfos[0].Id;
                            result = await LoginHelper.GetRealmKey(zoneScene);
                            if (result == ErrorCode.ERR_Success)
                            {
                                await LoginHelper.EnterGame(zoneScene);
                            }
                        }
                    }
                }
            }

            return result;
        }
    }
}