using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class AgeStageConfigCategory : ProtoObject, IMerge
    {
        public static AgeStageConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, AgeStageConfig> dict = new Dictionary<int, AgeStageConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<AgeStageConfig> list = new List<AgeStageConfig>();
		
        public AgeStageConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            AgeStageConfigCategory s = o as AgeStageConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (AgeStageConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public AgeStageConfig Get(int id)
        {
            this.dict.TryGetValue(id, out AgeStageConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (AgeStageConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, AgeStageConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<AgeStageConfig> GetList()
        {
            return this.list;
        }

        public AgeStageConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class AgeStageConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>年龄段</summary>
		[ProtoMember(2)]
		public string AgeStage { get; set; }
		/// <summary>起始年龄</summary>
		[ProtoMember(3)]
		public int StartAge { get; set; }
		/// <summary>终止年龄</summary>
		[ProtoMember(4)]
		public int EndAge { get; set; }
		/// <summary>劳力Min</summary>
		[ProtoMember(5)]
		public int LaborMin { get; set; }
		/// <summary>劳力Max</summary>
		[ProtoMember(6)]
		public int LaborMax { get; set; }
		/// <summary>劳力年龄衰减</summary>
		[ProtoMember(7)]
		public int LaborAttenuate { get; set; }
		/// <summary>随机速度最小值</summary>
		[ProtoMember(8)]
		public int SpeedMin { get; set; }
		/// <summary>随机速度最大值</summary>
		[ProtoMember(9)]
		public int SpeedMax { get; set; }

	}
}
