using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DistanceConfigCategory : ProtoObject, IMerge
    {
        public static DistanceConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DistanceConfig> dict = new Dictionary<int, DistanceConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DistanceConfig> list = new List<DistanceConfig>();
		
        public DistanceConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DistanceConfigCategory s = o as DistanceConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DistanceConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DistanceConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DistanceConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DistanceConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DistanceConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DistanceConfig> GetList()
        {
            return this.list;
        }

        public DistanceConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DistanceConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>节点1</summary>
		[ProtoMember(2)]
		public int Region1 { get; set; }
		/// <summary>节点2</summary>
		[ProtoMember(3)]
		public int Region2 { get; set; }
		/// <summary>距离</summary>
		[ProtoMember(4)]
		public int Distance { get; set; }

	}
}
