using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class LifeExpConfigCategory : ProtoObject, IMerge
    {
        public static LifeExpConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, LifeExpConfig> dict = new Dictionary<int, LifeExpConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<LifeExpConfig> list = new List<LifeExpConfig>();
		
        public LifeExpConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            LifeExpConfigCategory s = o as LifeExpConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (LifeExpConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public LifeExpConfig Get(int id)
        {
            this.dict.TryGetValue(id, out LifeExpConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (LifeExpConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, LifeExpConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<LifeExpConfig> GetList()
        {
            return this.list;
        }

        public LifeExpConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class LifeExpConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>寿命</summary>
		[ProtoMember(2)]
		public int Life { get; set; }
		/// <summary>寿命经验</summary>
		[ProtoMember(3)]
		public int LifeExp { get; set; }

	}
}
