using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class MedicineConfigCategory : ProtoObject, IMerge
    {
        public static MedicineConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, MedicineConfig> dict = new Dictionary<int, MedicineConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<MedicineConfig> list = new List<MedicineConfig>();
		
        public MedicineConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            MedicineConfigCategory s = o as MedicineConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (MedicineConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public MedicineConfig Get(int id)
        {
            this.dict.TryGetValue(id, out MedicineConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (MedicineConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, MedicineConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<MedicineConfig> GetList()
        {
            return this.list;
        }

        public MedicineConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class MedicineConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>药物名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>效果</summary>
		[ProtoMember(4)]
		public int Effection { get; set; }
		/// <summary>效果数值</summary>
		[ProtoMember(5)]
		public int EffectionValue { get; set; }
		/// <summary>治愈最大值</summary>
		[ProtoMember(6)]
		public int CureMax { get; set; }
		/// <summary>药效</summary>
		[ProtoMember(7)]
		public int Efficacy { get; set; }
		/// <summary>毒性</summary>
		[ProtoMember(8)]
		public int Toxicity { get; set; }
		/// <summary>持续时间</summary>
		[ProtoMember(9)]
		public int Duration { get; set; }

	}
}
