using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class MissionStepConfigCategory : ProtoObject, IMerge
    {
        public static MissionStepConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, MissionStepConfig> dict = new Dictionary<int, MissionStepConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<MissionStepConfig> list = new List<MissionStepConfig>();
		
        public MissionStepConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            MissionStepConfigCategory s = o as MissionStepConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (MissionStepConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public MissionStepConfig Get(int id)
        {
            this.dict.TryGetValue(id, out MissionStepConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (MissionStepConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, MissionStepConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<MissionStepConfig> GetList()
        {
            return this.list;
        }

        public MissionStepConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class MissionStepConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>目标类型</summary>
		[ProtoMember(2)]
		public int[] TargetType { get; set; }
		/// <summary>目标参数</summary>
		[ProtoMember(3)]
		public int[] TargetTypeParameter { get; set; }
		/// <summary>目标进度</summary>
		[ProtoMember(4)]
		public int[] TargetAmount { get; set; }
		/// <summary>附加条件</summary>
		[ProtoMember(5)]
		public int[] AdditionalCondition { get; set; }
		/// <summary>附加条件参数</summary>
		[ProtoMember(6)]
		public int[] AdditionalConditionParameter { get; set; }
		/// <summary>完成任务</summary>
		[ProtoMember(7)]
		public int Completed { get; set; }
		/// <summary>是否触发新任务</summary>
		[ProtoMember(8)]
		public int IsMissionTrigger { get; set; }

	}
}
