using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ProductionConfigCategory : ProtoObject, IMerge
    {
        public static ProductionConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ProductionConfig> dict = new Dictionary<int, ProductionConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ProductionConfig> list = new List<ProductionConfig>();
		
        public ProductionConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ProductionConfigCategory s = o as ProductionConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ProductionConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ProductionConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ProductionConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ProductionConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ProductionConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ProductionConfig> GetList()
        {
            return this.list;
        }

        public ProductionConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ProductionConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>产出类型</summary>
		[ProtoMember(2)]
		public int Type { get; set; }
		/// <summary>囤积上限</summary>
		[ProtoMember(3)]
		public int Volume { get; set; }
		/// <summary>产出间隔</summary>
		[ProtoMember(4)]
		public int OutInterval { get; set; }
		/// <summary>单次产出</summary>
		[ProtoMember(5)]
		public int EachOut { get; set; }
		/// <summary>基础产出物品</summary>
		[ProtoMember(6)]
		public int BaseOut { get; set; }
		/// <summary>额外产出物品</summary>
		[ProtoMember(7)]
		public int ExtraOut { get; set; }
		/// <summary>额外产出概率</summary>
		[ProtoMember(8)]
		public int ExtraOutWeight { get; set; }

	}
}
