using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class TechnologyConfigCategory : ProtoObject, IMerge
    {
        public static TechnologyConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, TechnologyConfig> dict = new Dictionary<int, TechnologyConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<TechnologyConfig> list = new List<TechnologyConfig>();
		
        public TechnologyConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            TechnologyConfigCategory s = o as TechnologyConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (TechnologyConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public TechnologyConfig Get(int id)
        {
            this.dict.TryGetValue(id, out TechnologyConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (TechnologyConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, TechnologyConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<TechnologyConfig> GetList()
        {
            return this.list;
        }

        public TechnologyConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class TechnologyConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>研究耗材</summary>
		[ProtoMember(3)]
		public int[] UnlockMatrial { get; set; }
		/// <summary>研究耗材数量</summary>
		[ProtoMember(4)]
		public int[] UnlockMatrialNum { get; set; }
		/// <summary>研究耗时</summary>
		[ProtoMember(5)]
		public int UnlockTime { get; set; }
		/// <summary>图标</summary>
		[ProtoMember(6)]
		public string Icon { get; set; }
		/// <summary>描述</summary>
		[ProtoMember(7)]
		public string Description { get; set; }

	}
}
