using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class UpExpConfigCategory : ProtoObject, IMerge
    {
        public static UpExpConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, UpExpConfig> dict = new Dictionary<int, UpExpConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<UpExpConfig> list = new List<UpExpConfig>();
		
        public UpExpConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            UpExpConfigCategory s = o as UpExpConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (UpExpConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public UpExpConfig Get(int id)
        {
            this.dict.TryGetValue(id, out UpExpConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (UpExpConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, UpExpConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<UpExpConfig> GetList()
        {
            return this.list;
        }

        public UpExpConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class UpExpConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>等级</summary>
		[ProtoMember(2)]
		public int Level { get; set; }
		/// <summary>升级经验</summary>
		[ProtoMember(3)]
		public int Exp { get; set; }
		/// <summary>生命增加</summary>
		[ProtoMember(4)]
		public int HPUp { get; set; }
		/// <summary>防御增加</summary>
		[ProtoMember(5)]
		public int DEFUp { get; set; }
		/// <summary>攻击增加</summary>
		[ProtoMember(6)]
		public int ATKUp { get; set; }

	}
}
