using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class WeaponTypeConfigCategory : ProtoObject, IMerge
    {
        public static WeaponTypeConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, WeaponTypeConfig> dict = new Dictionary<int, WeaponTypeConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<WeaponTypeConfig> list = new List<WeaponTypeConfig>();
		
        public WeaponTypeConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            WeaponTypeConfigCategory s = o as WeaponTypeConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (WeaponTypeConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public WeaponTypeConfig Get(int id)
        {
            this.dict.TryGetValue(id, out WeaponTypeConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (WeaponTypeConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, WeaponTypeConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<WeaponTypeConfig> GetList()
        {
            return this.list;
        }

        public WeaponTypeConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class WeaponTypeConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>武器名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>反击动作</summary>
		[ProtoMember(3)]
		public int FightBack { get; set; }
		/// <summary>追击动作</summary>
		[ProtoMember(4)]
		public int Chase { get; set; }
		/// <summary>招架动作</summary>
		[ProtoMember(5)]
		public int Parry { get; set; }
		/// <summary>武器描述</summary>
		[ProtoMember(6)]
		public string Describe { get; set; }

	}
}
