using System.Collections.Generic;
using System.IO;
using ET;
using FairyGUI;
using LitJson;
using UnityEditor;
using UnityEngine;
using UnityEngine.UIElements;
using PVSkill;

public class SkillEditor : EditorWindow
{
    private Button loadButton;
    private ListView skillFatherList;
    private SkillFatherElement skillFatherElement;
    


    [MenuItem("Window/UI Toolkit/SkillEditor")]
    public static void ShowExample()
    {
        SkillEditor wnd = GetWindow<SkillEditor>();
        wnd.titleContent = new GUIContent("SkillEditor");
    }
    
    private void LoadConfig<T>() where T: IConfigCategory
    {
        string jsonDir = Application.dataPath + "../../../Bin/Json/c";
        string categoryName = typeof (T).Name;
        string configName = categoryName.Substring(0, categoryName.Length - 8);
        string path = Path.Combine(jsonDir, $"{configName}.txt");
        string data = File.ReadAllText(path);
        var configCategory = JsonMapper.ToObject<T>(data);
        configCategory.Init();
    }

    public void CreateGUI()
    {
        // Each editor window contains a root VisualElement object
        VisualElement root = rootVisualElement;

        // VisualElements objects can contain other VisualElement following a tree hierarchy.
        // VisualElement label = new Label("Hello World! From C#");
        // root.Add(label);

        // Import UXML
        var visualTree = AssetDatabase.LoadAssetAtPath<VisualTreeAsset>("Assets/Editor/SkillEditor/SkillEditor.uxml");
        // VisualElement labelFromUXML = visualTree.Instantiate();
        // root.Add(labelFromUXML);
        visualTree.CloneTree(root);

        // A stylesheet can be added to a VisualElement.
        // The style will be applied to the VisualElement and all of its children.
        var styleSheet = AssetDatabase.LoadAssetAtPath<StyleSheet>("Assets/Editor/SkillEditor/SkillEditor.uss");
        root.styleSheets.Add(styleSheet);
        // VisualElement labelWithStyle = new Label("Hello World! With Style");
        // labelWithStyle.styleSheets.Add(styleSheet);
        // root.Add(labelWithStyle);

        skillFatherList= root.Q<ListView>("SkillFatherList");
        skillFatherList.makeItem = MakeSkillFatherListItem;
        skillFatherList.bindItem = this.BindSkillFatherListItem;
        skillFatherList.onSelectionChange += OnSkillFatherItemChosen;
        var skillFatherInfo = root.Q<VisualElement>("InfoPanel");
        skillFatherElement = new SkillFatherElement(skillFatherInfo);

        loadButton = root.Q<Button>("loadBtn");
        this.loadButton.clicked += this.OnLoadBtnClick;
    }



    private void OnSkillFatherItemChosen(IEnumerable<object> obj)
    {
        foreach (object o in obj)
        {
            SkillFatherConfig config = (SkillFatherConfig) o;
            this.skillFatherElement.SetDataByConfig(config);
        }
    }

    private void BindSkillFatherListItem(VisualElement ve, int index)
    {
        Label label = ve as Label;
        var config = SkillFatherConfigCategory.Instance.list[index];
        label.text = config.SkillName;
    }

    private VisualElement MakeSkillFatherListItem()
    {
        var label = new Label();
        label.AddToClassList("FatherSkillLabel");
        return label;
    }



    private void OnLoadBtnClick()
    {
        LoadConfig<SkillFatherConfigCategory>();
        this.LoadConfig<SkillSonConfigCategory>();
        this.LoadConfig<SkillUpConfigCategory>();
        this.LoadConfig<SkillBuffConfigCategory>();
        this.LoadConfig<SkillEffectConfigCategory>();
        
        this.skillFatherList.itemsSource = SkillFatherConfigCategory.Instance.list;
        this.skillFatherList.RefreshItems();
        
        // load fui data
        UIPackage.AddPackage("Assets/Bundles/FUI/SkillIcon");
    }
}