﻿using ET.EventType;
using UnityEngine;

namespace ET
{
    using FairyGUI;
    using FUIMain;
    using FUIPeople;
    using FUIStoreHouse;
    using System;
    using System.Text;

    [Timer(TimerType.UpdateGameTimeTimer)]
    public class UpdateGameTimeTimer : ATimer<FUIMainComponent>
    {
        public override void Run(FUIMainComponent self)
        {
            try
            {
                if (self.IsDisposed || self.Parent == null) return;
                if (self.DomainScene() == null) return;
                self.UpdateGameTime().Coroutine();
            }
            catch(Exception ex)
            {
                Log.Error(ex.ToString());
            }
        }
    }


    public class FUIMainComponentSystemAwakeSystem : AwakeSystem<FUIMainComponent>
    {
        public override void Awake(FUIMainComponent self)
        {
            self.Awake();
        }
    }
    

    [FriendClass(typeof(Unit))]
    [FriendClass(typeof(FUI_DayNightTime))]
    [FriendClass(typeof(FUI_CloseBtn))]
    public static partial class FUIMainComponentSystem
    {
        public static void Awake(this FUIMainComponent self)
        {
            self.FUIMain = self.Parent.GetComponent<FUI_Main>();
            self.FUIMain.FGComp.fairyBatching = true;

            // 加载所有Item资源
            self.ZoneScene().GetComponent<FUIPackageComponent>().AddPackage(FUIPackage.AllItem);

            self.FUIMain.m_scaleBtn.FGComp.onClick.Set(() => 
            {
                self.OpenScaleInfo();
            });
            self.FUIMain.m_SeasonBtn.FGComp.onClick.Set(() => 
            {
                self.OpenSeasonInfo();
            });
            self.FUIMain.m_BuildingButton.FGComp.onClick.Set(() =>
            {
                self.OnBuildingButtonClick();
            });
            
            self.FUIMain.m_PeopleButton.FGComp.onClick.Set(() =>
            {
                self.OnPeopleButtonClick();
            });
            
            self.FUIMain.m_PartnerButton.FGComp.onClick.Set(() =>
            {
                self.OnPartnerButtonClick();
            });
            
            self.FUIMain.m_StoreHouseButton.FGComp.onClick.Set(() =>
            {
                self.OnStoreHouseButtonClick();
            });
            
            self.FUIMain.m_LineupButton.FGComp.onClick.Set(() =>
            {
                self.OnLineupButtonClick();
            });

            
            self.FUIMain.m_TestButton.FGComp.onClick.Set(() =>
            {
                self.OnTestButtonClick();
            });

            self.FUIMain.m_SynthesisButton.FGComp.onClick.Set(() =>
            {
                self.OnSynthesisButtonClick();
            });

            self.FUIMain.m_NightEventWindow.m_closeBtn.FGComp.onClick.Set(() => 
            {
                self.FUIMain.m_NightEventWindow.Visible = false;
            });

            self.FUIMain.m_NightReview.FGComp.onClick.Set(() =>
            {
                Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
                UnitHelper.GetNightEvent(unit).Coroutine();
            });

            TimerComponent.Instance.Remove(ref self.UpdateTimer);
            self.UpdateTimer = TimerComponent.Instance.NewRepeatedTimer(1000, TimerType.UpdateGameTimeTimer, self);

            self.RefreshValleyInfo();
        }

        public static void OnSynthesisButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            var win = fuiComponent.ShowWindow<FUISynthesis.FUI_SynthesisWindow, FUISynthesisWindowComponent>();
            win.ShowInfo(-1);
        }

        public static void RefreshValleyInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            int scale = unit.Scale;

            self.FUIMain.m_PlayerName.text = unit.Name;
            self.FUIMain.m_LevelName.text = ScaleConfigCategory.Instance.Get(Mathf.Max(1, scale))?.ScaleName;
            self.RefreshFoodInfo();
            self.RefreshWaterInfo();
            self.RefreshSeasonInfo();
            self.RefreshTimeInfo();
            self.FUIMain.m_moneyNum.text = unit.SilverTael.ToString();
            self.FUIMain.m_goldNum.text = unit.GoldIngot.ToString();
        }

        private static void RefreshSeasonInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var seasonBtn = self.FUIMain.m_SeasonBtn;
            int season = unit.Season;
            SeasonConfig seasonConfig = SeasonConfigCategory.Instance.Get(season);
            int seasonDay = WorldParametersConfigCategory.Instance.Get(WorldParam.SeasonDay).Value[0];

            string seasonName = seasonConfig.Name;
            string fColor = string.Empty;
            string bColor = string.Empty;
            switch (season)
            {
                case 1:
                    fColor = "#CC0000";
                    bColor = "#00FF00";
                    break;
                case 2:
                    fColor = "#00FF00";
                    bColor = "#FF6600";
                    break;
                case 3:
                    fColor = "#FF6600";
                    bColor = "#0066FF";
                    break;
                case 4:
                    fColor = "#0066FF";
                    bColor = "#CC0000";
                    break;
            }

            ColorUtility.TryParseHtmlString(fColor, out Color frontcolor);
            seasonBtn.m_Front.color = frontcolor;
            ColorUtility.TryParseHtmlString(bColor, out Color backColor);
            seasonBtn.m_Back.color = backColor;
            seasonBtn.m_Name.text = seasonName;
            int seasonDayRemain = seasonDay - (unit.Day - 1) % seasonDay;

            float fillAmount = (float)(seasonDayRemain) / seasonDay;
            seasonBtn.m_Front.fillAmount = 1 - fillAmount; 
        }

        public static async ETTask UpdateGameTime(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            await UnitHelper.GetUnitGameTime(unit);
            self.RefreshTimeInfo();
        }

        private static void RefreshTimeInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var com = self.FUIMain.m_ScrollPane;
            int time = unit.GameTime;
            int hour = time / 100;
            int minute = time % 100;
            float pgs = hour - 1 + (float)minute / 60;
            var tile1 = com.m_Tile1;
            var tile2 = com.m_Tile2;
            var pgsFullWidth = tile1.FGComp.width;
            float initPos = 27;
            float interval = pgsFullWidth / 24;

            var x = initPos - interval * pgs;
            tile1.FGComp.position = new Vector3(x, 0, 0);
            if(hour >= 4 && hour < 9)
            {
                tile2.FGComp.position = new Vector3(x - pgsFullWidth, 0, 0);
            }
            else
            {
                tile2.FGComp.position = new Vector3(x + pgsFullWidth, 0, 0);
            }
            
            Game.EventSystem.Publish(new GameTime(){ Unit = unit, Hour = hour, Minute = minute});
        }

        private static void OpenSeasonInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var com = self.FUIMain.m_SeasonDetail;
            var seasonBtn = com.m_SeasonBtn;
            int season = unit.Season;
            SeasonConfig seasonConfig = SeasonConfigCategory.Instance.Get(season);
            int seasonDay = WorldParametersConfigCategory.Instance.Get(WorldParam.SeasonDay).Value[0];


            string seasonName = seasonConfig.Name; ;
            string fColor = string.Empty;
            string bColor = string.Empty;
            switch (season)
            {
                case 1:
                    fColor = "#CC0000";
                    bColor = "#00FF00";
                    break;
                case 2:
                    fColor = "#00FF00";
                    bColor = "#FF6600";
                    break;
                case 3:
                    fColor = "#FF6600";
                    bColor = "#0066FF";
                    break;
                case 4:
                    fColor = "#0066FF";
                    bColor = "#CC0000";
                    break;
            }

            ColorUtility.TryParseHtmlString(fColor, out Color frontcolor);
            seasonBtn.m_Front.color = frontcolor;
            ColorUtility.TryParseHtmlString(bColor, out Color backColor);
            seasonBtn.m_Back.color = backColor;
            seasonBtn.m_Name.text = seasonName;

            int seasonDayRemain = seasonDay - (unit.Day - 1) % seasonDay;

            float fillAmount = seasonDayRemain / (float)seasonDay;
            seasonBtn.m_Front.fillAmount = 1 - fillAmount;

            com.m_closeBtn.FGComp.onClick.Set(() => { com.Visible = false; });
            com.m_SeasonChangeRemainTxt.SetVar("value", (seasonDayRemain).ToString()).FlushVars();
            var crops = CropConfigCategory.Instance.GetAllSeasonCrop(season);
            com.m_ItemList.RemoveChildrenToPool();
            foreach(var c in crops)
            {
                CropConfig cropConfig = CropConfigCategory.Instance.Get(c);
                var slot = com.m_ItemList.AddItemFromPool().asButton;
                string iconUrl = $"{cropConfig.CropPic}";
                Log.Debug("Icon: " + iconUrl);
                slot.GetChild("Icon").asLoader.url = iconUrl;
            }
            com.m_SeasonDescription.text = seasonConfig.Description;
            com.Visible = true;
            com.FGComp.Center();
        }

        public static void OpenScaleInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var com = self.FUIMain.m_ValleyScale;
            int scale = unit.Scale;
            var scaleConfig = ScaleConfigCategory.Instance.Get(Mathf.Max(1, scale));
            var nextScale = ScaleConfigCategory.Instance.Get(Mathf.Max(1, scale) + 1);
            com.m_LevelName.text = scaleConfig?.ScaleName;
            //TODO - 规模图片
            com.m_scaleIcon.url = $"ui://AllItem/AllItem4001";
            var pgs = com.m_scalePgs.FGComp;
            pgs.max = nextScale.Prosperity;
            pgs.value = unit.Prosperity;
            com.m_scalePgs.m_value.text = $"{unit.Prosperity}/{nextScale.Prosperity}";
            com.m_upgradeBtn.FGComp.onClick.Set(self.UpgradeValleyScale);
            com.m_closeBtn.FGComp.onClick.Set(() => { com.Visible = false; });

            //设置解锁建筑列表
            var buildings = SynthesisConfigCategory.Instance.GetAllBuildingsWithScaleLimit(unit.Scale + 1);
            com.m_BuildingSlots.RemoveChildrenToPool();
            foreach (var building in buildings)
            {
                var slotItem = com.m_BuildingSlots.AddItemFromPool().asButton;
                var buildingConfig = StructureConfigCategory.Instance.Get(building);
                string iconUrl = $"{buildingConfig.Picture}";
                slotItem.GetChild("Icon").asLoader.url = iconUrl;
            }

            //设置奖励道具列表
            int rewardGroup = nextScale.RewardGroup;
            if (rewardGroup != 0)
            {
                RewardGroupConfig rewardGroupConfig = RewardGroupConfigCategory.Instance.Get(rewardGroup);
                int[] itemIds = rewardGroupConfig.ItemId;
                int[] itemNums = rewardGroupConfig.Number;
                com.m_unlockSlots.RemoveChildrenToPool();
                for(int i=0; i < itemIds.Length; i++)
                {
                    int itemID = itemIds[i];
                    int itemNum = itemNums[i];
                    var slotItem = com.m_unlockSlots.AddItemFromPool().asButton;
                    AllItemConfig allItemConfig = AllItemConfigCategory.Instance.Get(itemID);
                    
                    slotItem.GetChild("Icon").asLoader.url = $"ui://AllItem/{allItemConfig.Icon}";
                    slotItem.GetChild("Num").asTextField.text = $"{itemNum}";
                }
            }
            
            com.Visible = true;
            com.FGComp.Center();
        }

        private static async void UpgradeValleyScale(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            int scale = unit.Scale;
            int prosperity = unit.Prosperity;
            var scaleConfig = ScaleConfigCategory.Instance.Get(Mathf.Max(1, scale));
            if(prosperity < scaleConfig.Prosperity)
            {
                CommonHelper.ShowHint(self.ZoneScene(), "桃谷等级繁荣度不足,无法升级桃谷规模");
                return;
            }
            int result = await UnitHelper.UpgradeScale(unit);
            if (result != ErrorCode.ERR_Success) return;
            self.RefreshValleyInfo();
        }

        //TODO - 需要给Food添加一个委托,方便控制Food值发生变化时直接响应界面信息刷新
        public static void RefreshFoodInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var foodPgs = self.FUIMain.m_foodPgs.FGComp;
            var foodText = self.FUIMain.m_foodPgs.m_value;
            var foodBar = self.FUIMain.m_foodPgs.m_bar;
            //当前食物总和
            var foodSum = unit.Food;
            //人口
            var peopleNum = Mathf.Max(1, unit.GetComponent<PeopleComponent>().Children.Count);
            //分钟消耗
            var minuteCost = WorldParametersConfigCategory.Instance.Get(WorldParam.FoodConsumption).Value[0];
            //日消耗
            var dayCost = peopleNum * minuteCost * 12 * 5;

            int foodPgsValue = double.IsNaN(foodPgs.value) ? 0 : (int)foodPgs.value;
            if(foodSum < 0)
            {
                ColorUtility.TryParseHtmlString("#FF9900", out Color newcolor);
                foodBar.color = newcolor;
                foodPgs.value = foodPgs.max;
                foodText.text = $"食物不足";
            }
            else if (foodSum <= 2 * dayCost)
            {
                ColorUtility.TryParseHtmlString("#00FF00", out Color newcolor);
                foodBar.color = newcolor;
                foodPgs.max = dayCost;
                foodPgs.value = Mathf.Min(foodSum, dayCost);
                foodText.text = $"{foodSum}/{dayCost}";
            }
            else if(foodSum > 2 * dayCost)
            {
                ColorUtility.TryParseHtmlString("#00FF00", out Color newcolor);
                foodBar.color = newcolor;
                foodText.text = $"{foodSum}";
            }

            foodPgs.onRollOver.Set(self.ShowFoodStatePopup);
            foodPgs.onRollOut.Set(self.HidePopup);
        }

        public static void ShowFoodStatePopup(this FUIMainComponent self, EventContext context)
        {
            GComponent foodState = context.sender as GComponent;
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var popupCom = self.FUIMain.m_ContentPopup;
            StringBuilder sb = new StringBuilder();
            //人口
            var peopleNum = Mathf.Max(1, unit.GetComponent<PeopleComponent>().Children.Count);
            //分钟消耗
            var minuteCost = WorldParametersConfigCategory.Instance.Get(WorldParam.FoodConsumption).Value[0];
            //日消耗
            var dayCost = peopleNum * minuteCost * 12 * 5;
            sb.Clear();
            if(unit.Food < 0)
            {
                sb.AppendLine($"当前:{unit.Food}/{dayCost}");
            }
            sb.AppendLine($"变化:{0}/分钟");
            sb.AppendLine($"储备:可供{unit.Food/dayCost}天");
            popupCom.m_Value.text = sb.ToString();

            popupCom.FGComp.visible = true;
            GRoot.inst.ShowPopup(self.FUIMain.m_ContentPopup.FGComp, foodState);
        }

        public static void HidePopup(this FUIMainComponent self)
        {
            var popupCom = self.FUIMain.m_ContentPopup;
            popupCom.Visible = false;
        }

        //TODO - 需要给Water添加一个委托,方便控制Water值发生变化时直接响应界面信息刷新
        public static void RefreshWaterInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var waterPgs = self.FUIMain.m_waterPgs.FGComp;
            var waterText = self.FUIMain.m_waterPgs.m_value;
            var waterBar = self.FUIMain.m_waterPgs.m_bar;

            //当前水总和
            var waterSum = unit.Water;
            //人口
            var peopleNum = Mathf.Max(1, unit.GetComponent<PeopleComponent>().Children.Count);
            //分钟消耗
            var minuteCost = WorldParametersConfigCategory.Instance.Get(WorldParam.WaterConsumption).Value[0];
            //日消耗
            var dayCost = peopleNum * minuteCost * 12 * 5;

            int waterPgsValue = double.IsNaN(waterPgs.value) ? 0 : (int)waterPgs.value;
            if (waterSum < 0)
            {
                ColorUtility.TryParseHtmlString("#CC0000", out Color newcolor);
                waterBar.color = newcolor;
                waterPgs.value = waterPgs.max;
                waterText.text = $"水量不足";
            }
            else if (waterSum <= 2 * dayCost)
            {
                ColorUtility.TryParseHtmlString("#00FF00", out Color newcolor);
                waterBar.color = newcolor;
                waterPgs.max = dayCost;
                waterPgs.value = Mathf.Min(waterSum, dayCost);
                waterText.text = $"{waterSum}/{dayCost}";
            }
            else if (waterSum > 2 * dayCost)
            {
                ColorUtility.TryParseHtmlString("#00FF00", out Color newcolor);
                waterBar.color = newcolor;
                waterText.text = $"{waterSum}";
            }

            waterPgs.onRollOver.Set(self.ShowWaterStatePopup);
            waterPgs.onRollOut.Set(self.HidePopup);
        }

        public static void ShowWaterStatePopup(this FUIMainComponent self, EventContext context)
        {
            GComponent waterState = context.sender as GComponent;
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var popupCom = self.FUIMain.m_ContentPopup;
            StringBuilder sb = new StringBuilder();
            //人口
            var peopleNum = Mathf.Max(1, unit.GetComponent<PeopleComponent>().Children.Count);
            //分钟消耗
            var minuteCost = WorldParametersConfigCategory.Instance.Get(WorldParam.WaterConsumption).Value[0];
            //日消耗
            var dayCost = peopleNum * minuteCost * 12 * 5;
            sb.Clear();
            if (unit.Water < 0)
            {
                sb.AppendLine($"当前:{unit.Water}/{dayCost}");
            }
            sb.AppendLine($"变化:{0}/分钟");
            sb.AppendLine($"储备:可供{unit.Water / dayCost}天");
            popupCom.m_Value.text = sb.ToString();

            popupCom.FGComp.visible = true;
            GRoot.inst.ShowPopup(self.FUIMain.m_ContentPopup.FGComp, waterState);
        }

        public static void RefreshSliverTaelInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            self.FUIMain.m_moneyNum.text = unit.SilverTael.ToString();
        }

        public static void RefreshGoldIngotInfo(this FUIMainComponent self)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            self.FUIMain.m_goldNum.text = unit.GoldIngot.ToString();
        }

        public static void OnBuildingButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            fuiComponent.ShowWindowAsync<FUIBuilding.FUI_BuildingWindow, FUIBuildingWindowComponent>().Coroutine();
        }

        public static void OnPeopleButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            fuiComponent.ShowWindowAsync<FUI_PeopleListWindow, FUIPeopleListWindowComponent>().Coroutine();
        }

        public static void OnPartnerButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            fuiComponent.ShowWindowAsync<FUI_PartnerListWindow, FUIPartnerListWindowComponent>().Coroutine();
        }

        public static void OnStoreHouseButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            fuiComponent.ShowWindowAsync<FUI_StoreHouseWindow, FUIStoreHouseWindowComponent>().Coroutine();
        }

        public static void OnLineupButtonClick(this FUIMainComponent self)
        {
            var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            fuiComponent.ShowWindowAsync<FUI_LineupWindow, FUILineupWindowComponent>().Coroutine();
        }
        
        public static void ShowNightEvent(this FUIMainComponent self, bool showAni = true)
        {
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
            var com = self.FUIMain.m_NightEventWindow;
            com.FGComp.Center();
            com.Visible = true;
            com.m_NightPass.SetVar("value", (15 - unit.Day % 15).ToString()).FlushVars();
            if (showAni)
            {
                com.m_cutscene.Play(() => {
                    com.FGComp.TweenFade(0, 1).OnComplete(() => {
                        com.Visible = false;
                    });
                });
            }
            else
            {
                com.FGComp.alpha = 1;
            }

            void AddItem()
            {
                //TODO - 暂时使用假信息取代
                if (self.events == null) return;
                if (com.m_MessageList.numItems >= self.events.Count) return;
                var eventInfo = self.events[com.m_MessageList.numItems];
                var id = eventInfo.EventId;
                var eventConfig = EventConfigCategory.Instance.Get(id);
                StringBuilder sb = new StringBuilder(eventConfig.Description);
                //设置村民名称
                sb.Replace("{村民名称}", eventInfo.Name);

                if(eventInfo.Drops != null)
                {
                    StringBuilder itemInfo = new StringBuilder();
                    foreach (var itemID in eventInfo.Drops.Keys)
                    {
                        AllItemConfig config = AllItemConfigCategory.Instance.Get(itemID);
                        int num = eventInfo.Drops[itemID];
                        itemInfo.Append($"{num}个{config.Name}");
                    }
                    sb.Replace("{道具}", itemInfo.ToString());
                }

                var newItem = com.m_MessageList.AddItemFromPool().asLabel;
                newItem.title = sb.ToString();
                newItem.visible = true;
                if (showAni)
                {
                    var ani = newItem.GetTransition("tweenAdd");
                    ani?.Play(AddItem);
                }
                else
                {
                    newItem.alpha = 1;
                    AddItem();
                }
            }

            com.m_MessageList.RemoveChildrenToPool();
            AddItem();
        }

        // for test
        public static void SetTime(this FUIMainComponent self, int time)
        {
            int hour = time / 100;
            int minute = time % 100;
            //self.FUIMain.m_Time.text = hour.ToString("00") + ":" + minute.ToString("00");
        }

        public static void OnTestButtonClick(this FUIMainComponent self)
        {
            // TODO: Test Logic
            Debug.Log("Test Button Click");
            Unit unit = UnitHelper.GetMyUnitFromZoneScene(self.ZoneScene());
 
            BattleHelper.EnterBattle(unit, 1349401105754292243,ConstBattleType.BATTLE_PVE).Coroutine();
            //var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
            // fuiComponent.ShowWindowAsync<FUIBuilding.FUI_PrepareBuildingWindow, FUIPrepareBuildingWindowComponent>().Coroutine();
        }
    }
}