﻿using System;
using FairyGUI;

namespace ET
{
    using FUIMain;
    using FUIRole;
    public class FUIRoleComponentAwakeSystem : AwakeSystem<FUIRoleComponent>
    {
        public override void Awake(FUIRoleComponent self)
        {
            self.Awake();
        }
    }
    
    [FriendClass(typeof(RoleInfo))]
    [FriendClass(typeof(RoleInfosComponent))]
    public static partial class FUIRoleComponentSystem
    {
        public static void Awake(this FUIRoleComponent self)
        {
            self.FUIRole = self.Parent.GetComponent<FUI_Role>();
            UIObjectFactory.SetPackageItemExtension(FUIPackage.Role_RoleItem, typeof(RoleItem));
            
            self.FUIRole.m_CreateButton.FGComp.onClick.Set(()=>
            {
                self.OnCreateRoleButtonClick().Coroutine();
            });
            
            self.FUIRole.m_DeleteButton.FGComp.onClick.Set(() =>
            {
                self.OnDeleteRoleButtonClick().Coroutine();
            });
            
            self.FUIRole.m_EnterGameButton.FGComp.onClick.Set(() =>
            {
                self.OnEnterGameButtonClick().Coroutine();
            });
            
            self.Init();
            
#if CLIENTTEST
            self.OnEnterGameButtonClick().Coroutine();
#endif
        }

        public static void Init(this FUIRoleComponent self)
        {
            self.RefreshRoleItems();
        }

        public static void RefreshRoleItems(this FUIRoleComponent self)
        {
            var roleInfosComp = self.ZoneScene().GetComponent<RoleInfosComponent>();
            var roleInfos = roleInfosComp.RoleInfos;
            var list = self.FUIRole.m_RoleList;
            list.RemoveChildren();
            foreach (var roleInfo in roleInfos)
            {
                RoleItem item = (RoleItem) list.AddItemFromPool();
                item.setName(roleInfo.Name);
                item.data = roleInfo.Id;
                item.onClick.Add(self.OnRoleItemClick);
            }
            
            if (roleInfosComp.RoleInfos.Count > 0)
            {
                roleInfosComp.CurRoleId = roleInfosComp.RoleInfos[0].Id;
                list.AddSelection(0, true);
            }
        }

        public static void OnRoleItemClick(this FUIRoleComponent self, EventContext context)
        {
            long roleId = (long) (context.sender as GObject).data;
            var roleInfosComp = self.ZoneScene().GetComponent<RoleInfosComponent>();
            roleInfosComp.CurRoleId = roleId;
        }

        public static async ETTask OnCreateRoleButtonClick(this FUIRoleComponent self)
        {
            string name = self.FUIRole.m_NameInput.text;

            if (string.IsNullOrEmpty(name))
            {
                Log.Error("Name is null");
                return;
            }

            try
            {
                int errorCode = await LoginHelper.CreateRole(self.ZoneScene(), name);
                if (errorCode != ErrorCode.ERR_Success)
                {
                    Log.Error(errorCode.ToString());
                    return;
                }
				
                self.RefreshRoleItems();
            }
            catch (Exception e)
            {
                Log.Error(e.ToString());
                return;
            }
        }
        
        public static async ETTask OnDeleteRoleButtonClick(this FUIRoleComponent self)
        {
            var roleInfosComp = self.ZoneScene().GetComponent<RoleInfosComponent>();
            if (roleInfosComp.CurRoleId == 0)
            {
                Log.Error("请选择需要删除的角色");
                return;
            }

            try
            {
                int errorCode = await LoginHelper.DeleteRole(self.ZoneScene());
                if (errorCode != ErrorCode.ERR_Success)
                {
                    Log.Error(errorCode.ToString());
                    return;
                }
				
                self.RefreshRoleItems();
            }
            catch (Exception e)
            {
                Log.Error(e.ToString());
                return;
            }

            await ETTask.CompletedTask;
        }

        public static async ETTask OnEnterGameButtonClick(this FUIRoleComponent self)
        {
            if (self.ZoneScene().GetComponent<RoleInfosComponent>().CurRoleId == 0)
            {
                Log.Error("请选择要进行游戏的角色");
                return;
            }

            try
            {
                int errorCode = await LoginHelper.GetRealmKey(self.ZoneScene());
                if (errorCode != ErrorCode.ERR_Success)
                {
                    Log.Error(errorCode.ToString());
                    return;
                }


                errorCode = await LoginHelper.EnterGame(self.ZoneScene());
                if (errorCode != ErrorCode.ERR_Success)
                {
                    Log.Error(errorCode.ToString());
                    return;
                }
                
                await self.ZoneScene().GetComponent<ObjectWait>().Wait<WaitType.Wait_SceneChangeFinish>();
                var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
                fuiComponent.RemoveUI(FUI_Role.UIResName);
                await fuiComponent.ShowUIAsync<FUI_Main, FUIMainComponent>(FUIPackage.Main);
            }
            catch (Exception e)
            {
                Log.Error(e.ToString());
                return;
            }
        }
    }
}