﻿using System;
using System.Collections.Generic;
using FairyGUI;

namespace ET
{
    using FUIServer;
    using FUIRole;
    public class FUIServerComponentAwakeSystem : AwakeSystem<FUIServerComponent>
    {
        public override void Awake(FUIServerComponent self)
        {
            self.Awake();
        }
    }
    
    public class FUIServerComponentDestroySystem : DestroySystem<FUIServerComponent>
    {
        public override void Destroy(FUIServerComponent self)
        {
            
        }
    }

    [FriendClass(typeof(FUIServerComponent))]
    [FriendClass(typeof(ServerInfosComponent))]
    [FriendClass(typeof(ServerInfo))]
    public static partial class FUIServerComponentSystem
    {
        public static void Awake(this FUIServerComponent self)
        {
            self.FUIServer = self.Parent.GetComponent<FUI_Server>();
            UIObjectFactory.SetPackageItemExtension(FUIPackage.Server_ServerItem, typeof(ServerItem));
            
            self.FUIServer.m_EnterServer.FGComp.onClick.Set(() =>
            {
                self.OnEnterServerButtonClick().Coroutine();
            });
        }

        public static async ETTask Init(this FUIServerComponent self)
        {
            int result = await LoginHelper.GetServerInfos(self.ZoneScene());
            if (result != ErrorCode.ERR_Success)
            {
                Log.Error("Can't get server info");
                return;
            }
            
            
			
            var serverInfosComp = self.ZoneScene().GetComponent<ServerInfosComponent>();
            List<ServerInfo> serverInfos = serverInfosComp.ServerInfoList;
            
            var list = self.FUIServer.m_ServerList;
            for (int i = 0; i < serverInfos.Count; i++)
            {
                ServerInfo info = serverInfos[i];
                ServerItem item = (ServerItem) list.AddItemFromPool();
                item.SetName(info.ServerName);
                item.data = info.Id;
                item.onClick.Add(
                    self.OnServerItemClick
                );
            }
            
            // 默认选择第一个
            if (serverInfos.Count > 0)
            {
                serverInfosComp.CurSelectedServerId = (int)serverInfos[0].Id;
                list.AddSelection(0, true);
            }
            
#if CLIENTTEST
            self.OnEnterServerButtonClick().Coroutine();
#endif
        }

        public static void RenderListItem(this FUIServerComponent self, int index, GObject obj)
        {
            
        }

        public static void OnServerItemClick(this FUIServerComponent self, EventContext context)
        {
            var obj = context.sender as GObject;
            long serverId = (long) obj.data;
            var serverInfosComp = self.ZoneScene().GetComponent<ServerInfosComponent>();
            serverInfosComp.CurSelectedServerId = (int)serverId;
        }

        public static async ETTask OnEnterServerButtonClick(this FUIServerComponent self)
        {
            if (self.ZoneScene().GetComponent<ServerInfosComponent>().CurSelectedServerId == 0)
            {
                Log.Error("请选择区服");
                return;
            }

            try
            {
                int errorCode = await LoginHelper.GetRoles(self.ZoneScene());
                if (errorCode != ErrorCode.ERR_Success)
                {
                    Log.Error(errorCode.ToString());
                    return;
                }
				
                var fuiComponent = self.ZoneScene().GetComponent<FUIComponent>();
                fuiComponent.RemoveUI(FUI_Server.UIResName);
                await fuiComponent.ShowUIAsync<FUI_Role, FUIRoleComponent>(FUIPackage.Role);
            }
            catch (Exception e)
            {
                Log.Error(e.ToString());
            }
        }
    }
}