﻿using System.Collections.Generic;
using UnityEngine;

namespace ET
{
    namespace EventType
    {
        public struct AppStart
        {
        }

        public struct SceneChangeStart
        {
            public Scene ZoneScene;
        }
        
        
        public struct SceneChangeFinish
        {
            public Scene ZoneScene;
            public Scene CurrentScene;
        }

        public class ChangePosition: DisposeObject
        {
            public static readonly ChangePosition Instance = new ChangePosition();
            
            public Unit Unit;
            public WrapVector3 OldPos = new WrapVector3();

            // 因为是重复利用的，所以用完PublishClass会调用Dispose
            public override void Dispose()
            {
                this.Unit = null;
            }
        }

        public class ChangeRotation: DisposeObject
        {
            public static readonly ChangeRotation Instance = new ChangeRotation();
            
            public Unit Unit;
            
            // 因为是重复利用的，所以用完PublishClass会调用Dispose
            public override void Dispose()
            {
                this.Unit = null;
            }
        }

        public struct PingChange
        {
            public Scene ZoneScene;
            public long Ping;
        }
        
        public struct AfterCreateZoneScene
        {
            public Scene ZoneScene;
        }
        
        public struct AfterCreateCurrentScene
        {
            public Scene CurrentScene;
        }
        
        public struct AfterCreateLoginScene
        {
            public Scene LoginScene;
        }

        public struct AppStartInitFinish
        {
            public Scene ZoneScene;
        }

        public struct LoginFinish
        {
            public Scene ZoneScene;
        }

        public struct LoadingBegin
        {
            public Scene Scene;
        }

        public struct LoadingFinish
        {
            public Scene Scene;
        }

        public struct EnterMapFinish
        {
            public Scene ZoneScene;
        }

        public struct AfterUnitCreate
        {
            public Unit Unit;
        }
        
        public struct MoveStart
        {
            public Unit Unit;
        }

        public struct MoveStop
        {
            public Unit Unit;
        }

        public struct UnitEnterSightRange
        {
        }

        public struct AfterCreateBuilding
        {
            public Unit Unit;
            public Building Building;
            public bool IsNew;  // 是否是新建的，区别于初始化时的创建
        }
        
        public struct AfterRemoveBuilding
        {
            public Unit Unit;
            public Building Building;
        }

        public struct AfterCreateConstruct
        {
            public Unit Unit;
            public Construct Construct;
            public bool IsNew;  // 是否是新建的，区别于初始化时的创建
        }
        
        public struct AfterRemoveConstruct
        {
            public Unit Unit;
            public Construct Construct;
        }

        public struct AfterCreatePeople
        {
            public Unit Unit;
            public People People;
        }

        public struct AfterCreateResource
        {
            public Unit Unit;
            public ResourcePoint Resource;
        }

        public struct SyncTime
        {
            public Unit Unit;
            public int Time;
        }

        public struct CameraPosChange
        {
            public Scene ZoneScene;
        }

        public struct PeopleMoveStart
        {
            public People People;
        }

        public struct PeopleMoveEnd
        {
            public People People;
        }

        public struct SynthesisFinish
        {
            public Unit Unit;
            public int SynthesisID;
        }

        public struct CookingFinish
        {
            public Unit Unit;
            public int MixtureID;
            public int SelectedCuisine;
        }

        public struct CookingGameStart
        {
            public Unit Unit;
            public int SynthesisID;
        }
        
        public struct ShowNightEvent
        {
            public Unit Unit;
            public List<NightEvent> Events;
            public bool ShowAni;
        }

        public struct UpdateFood
        {
            public Unit Unit;
            public long Food;
        }

        public struct UpdateWater
        {
            public Unit Unit;
            public long Water;
        }

        public struct BattleStart
        {
            public Scene CurrentScene;
            public Battle BattleInfo;
        }

        public struct AttackRound
        {
            public Scene ZoneScene;
            public TurnInfo TurnInfo;
        }

        public struct ResourceMonsterKilled
        {
            public Unit Unit;
            public long ResourceId;
        }

        public struct BattleUnitDrop
        {
            public Unit Unit;
            public long BattleUnitId;
            public int ItemId;
            public int ItemNum;
        }

        public struct BattleUnitDie
        {
            public Unit Unit;
            public long BattleUnitId;
        }

        public struct BehaveTypeChanged
        {
            public People People;
        }

        public struct FarmlandStateChanged
        {
            public Farmland Farmland;
        }
    }
}