using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class AgeConfigCategory : ProtoObject, IMerge
    {
        public static AgeConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, AgeConfig> dict = new Dictionary<int, AgeConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<AgeConfig> list = new List<AgeConfig>();
		
        public AgeConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            AgeConfigCategory s = o as AgeConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (AgeConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public AgeConfig Get(int id)
        {
            this.dict.TryGetValue(id, out AgeConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (AgeConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, AgeConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<AgeConfig> GetList()
        {
            return this.list;
        }

        public AgeConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class AgeConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>年龄</summary>
		[ProtoMember(2)]
		public int Aage { get; set; }
		/// <summary>基础劳力</summary>
		[ProtoMember(3)]
		public int BasicLabor { get; set; }
		/// <summary>差劳力</summary>
		[ProtoMember(4)]
		public int PoorLabor { get; set; }
		/// <summary>中劳力</summary>
		[ProtoMember(5)]
		public int MediumLabor { get; set; }
		/// <summary>良劳力</summary>
		[ProtoMember(6)]
		public int GoodLabor { get; set; }
		/// <summary>优劳力</summary>
		[ProtoMember(7)]
		public int AdmirableBLabor { get; set; }
		/// <summary>极劳力</summary>
		[ProtoMember(8)]
		public int SuperLabor { get; set; }
		/// <summary>基础智慧</summary>
		[ProtoMember(9)]
		public int BasicWisdom { get; set; }
		/// <summary>差智慧</summary>
		[ProtoMember(10)]
		public int PoorWisdom { get; set; }
		/// <summary>中智慧</summary>
		[ProtoMember(11)]
		public int MediumWisdom { get; set; }
		/// <summary>良智慧</summary>
		[ProtoMember(12)]
		public int GoodWisdom { get; set; }
		/// <summary>优智慧</summary>
		[ProtoMember(13)]
		public int AdmirableWisdom { get; set; }
		/// <summary>极智慧</summary>
		[ProtoMember(14)]
		public int SuperWisdom { get; set; }

	}
}
