using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DialogueConfigCategory : ProtoObject, IMerge
    {
        public static DialogueConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DialogueConfig> dict = new Dictionary<int, DialogueConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DialogueConfig> list = new List<DialogueConfig>();
		
        public DialogueConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DialogueConfigCategory s = o as DialogueConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DialogueConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DialogueConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DialogueConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DialogueConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DialogueConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DialogueConfig> GetList()
        {
            return this.list;
        }

        public DialogueConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DialogueConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>对白内容</summary>
		[ProtoMember(2)]
		public string DialogueText { get; set; }

	}
}
