using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DialogueOptionConfigCategory : ProtoObject, IMerge
    {
        public static DialogueOptionConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DialogueOptionConfig> dict = new Dictionary<int, DialogueOptionConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DialogueOptionConfig> list = new List<DialogueOptionConfig>();
		
        public DialogueOptionConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DialogueOptionConfigCategory s = o as DialogueOptionConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DialogueOptionConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DialogueOptionConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DialogueOptionConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DialogueOptionConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DialogueOptionConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DialogueOptionConfig> GetList()
        {
            return this.list;
        }

        public DialogueOptionConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DialogueOptionConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>初始对白</summary>
		[ProtoMember(2)]
		public int InitialDialogue { get; set; }
		/// <summary>选项名</summary>
		[ProtoMember(3)]
		public string OptionName { get; set; }
		/// <summary>选项类型</summary>
		[ProtoMember(4)]
		public int OptionType { get; set; }
		/// <summary>类型参数</summary>
		[ProtoMember(5)]
		public int OptionTypeParameter { get; set; }
		/// <summary>类型参数2</summary>
		[ProtoMember(6)]
		public int[] RelatedParameter_1 { get; set; }
		/// <summary>下一对白</summary>
		[ProtoMember(7)]
		public int NextDialogue { get; set; }

	}
}
