using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ExtraOutputConfigCategory : ProtoObject, IMerge
    {
        public static ExtraOutputConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ExtraOutputConfig> dict = new Dictionary<int, ExtraOutputConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ExtraOutputConfig> list = new List<ExtraOutputConfig>();
		
        public ExtraOutputConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ExtraOutputConfigCategory s = o as ExtraOutputConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ExtraOutputConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ExtraOutputConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ExtraOutputConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ExtraOutputConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ExtraOutputConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ExtraOutputConfig> GetList()
        {
            return this.list;
        }

        public ExtraOutputConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ExtraOutputConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>物品编号</summary>
		[ProtoMember(2)]
		public int ItemId { get; set; }
		/// <summary>掉量区间</summary>
		[ProtoMember(3)]
		public int[] DropAmount { get; set; }
		/// <summary>掉量描述</summary>
		[ProtoMember(4)]
		public int DropDescription { get; set; }

	}
}
