using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class FightEffectConfigCategory : ProtoObject, IMerge
    {
        public static FightEffectConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, FightEffectConfig> dict = new Dictionary<int, FightEffectConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<FightEffectConfig> list = new List<FightEffectConfig>();
		
        public FightEffectConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            FightEffectConfigCategory s = o as FightEffectConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (FightEffectConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public FightEffectConfig Get(int id)
        {
            this.dict.TryGetValue(id, out FightEffectConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (FightEffectConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, FightEffectConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<FightEffectConfig> GetList()
        {
            return this.list;
        }

        public FightEffectConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class FightEffectConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>效果名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>作用对象</summary>
		[ProtoMember(3)]
		public int ActionObject { get; set; }
		/// <summary>持续类型</summary>
		[ProtoMember(4)]
		public int DurationType { get; set; }
		/// <summary>持续时间</summary>
		[ProtoMember(5)]
		public int Duration { get; set; }
		/// <summary>界面显示</summary>
		[ProtoMember(6)]
		public int InterfaceDisplay { get; set; }
		/// <summary>效果图标</summary>
		[ProtoMember(7)]
		public string BuffIcon { get; set; }
		/// <summary>效果描述</summary>
		[ProtoMember(8)]
		public string Describe { get; set; }

	}
}
