using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class FunctionItemConfigCategory : ProtoObject, IMerge
    {
        public static FunctionItemConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, FunctionItemConfig> dict = new Dictionary<int, FunctionItemConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<FunctionItemConfig> list = new List<FunctionItemConfig>();
		
        public FunctionItemConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            FunctionItemConfigCategory s = o as FunctionItemConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (FunctionItemConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public FunctionItemConfig Get(int id)
        {
            this.dict.TryGetValue(id, out FunctionItemConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (FunctionItemConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, FunctionItemConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<FunctionItemConfig> GetList()
        {
            return this.list;
        }

        public FunctionItemConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class FunctionItemConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>使用类型</summary>
		[ProtoMember(3)]
		public int UseType { get; set; }
		/// <summary>类型参数</summary>
		[ProtoMember(4)]
		public int UseTypeParameter { get; set; }

	}
}
