using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class LaborConfigCategory : ProtoObject, IMerge
    {
        public static LaborConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, LaborConfig> dict = new Dictionary<int, LaborConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<LaborConfig> list = new List<LaborConfig>();
		
        public LaborConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            LaborConfigCategory s = o as LaborConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (LaborConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public LaborConfig Get(int id)
        {
            this.dict.TryGetValue(id, out LaborConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (LaborConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, LaborConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<LaborConfig> GetList()
        {
            return this.list;
        }

        public LaborConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class LaborConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>劳力值</summary>
		[ProtoMember(2)]
		public int Labor { get; set; }
		/// <summary>劳力经验</summary>
		[ProtoMember(3)]
		public int LaborExp { get; set; }
		/// <summary>劳作收益系数</summary>
		[ProtoMember(4)]
		public int IncomeCoefficient { get; set; }

	}
}
