using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class MissionGroupConfigCategory : ProtoObject, IMerge
    {
        public static MissionGroupConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, MissionGroupConfig> dict = new Dictionary<int, MissionGroupConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<MissionGroupConfig> list = new List<MissionGroupConfig>();
		
        public MissionGroupConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            MissionGroupConfigCategory s = o as MissionGroupConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (MissionGroupConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public MissionGroupConfig Get(int id)
        {
            this.dict.TryGetValue(id, out MissionGroupConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (MissionGroupConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, MissionGroupConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<MissionGroupConfig> GetList()
        {
            return this.list;
        }

        public MissionGroupConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class MissionGroupConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>任务标题</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>任务类型</summary>
		[ProtoMember(3)]
		public string Type { get; set; }
		/// <summary>子任务组</summary>
		[ProtoMember(4)]
		public int[] ChildMissionGroup { get; set; }
		/// <summary>限时类型</summary>
		[ProtoMember(5)]
		public int TimeLimitType { get; set; }
		/// <summary>限时时间</summary>
		[ProtoMember(6)]
		public int TimeLimit { get; set; }
		/// <summary>可放弃</summary>
		[ProtoMember(7)]
		public int AbleAbandon { get; set; }
		/// <summary>接取次数</summary>
		[ProtoMember(8)]
		public int ReceiveTimes { get; set; }
		/// <summary>任务描述</summary>
		[ProtoMember(9)]
		public string MissionDescription { get; set; }

	}
}
