using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ScaleConfigCategory : ProtoObject, IMerge
    {
        public static ScaleConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ScaleConfig> dict = new Dictionary<int, ScaleConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ScaleConfig> list = new List<ScaleConfig>();
		
        public ScaleConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ScaleConfigCategory s = o as ScaleConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ScaleConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ScaleConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ScaleConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ScaleConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ScaleConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ScaleConfig> GetList()
        {
            return this.list;
        }

        public ScaleConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ScaleConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>规模名称</summary>
		[ProtoMember(2)]
		public string ScaleName { get; set; }
		/// <summary>繁荣度</summary>
		[ProtoMember(3)]
		public int Prosperity { get; set; }
		/// <summary>奖励组</summary>
		[ProtoMember(4)]
		public int RewardGroup { get; set; }

	}
}
