using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SkillConfigCategory : ProtoObject, IMerge
    {
        public static SkillConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SkillConfig> dict = new Dictionary<int, SkillConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SkillConfig> list = new List<SkillConfig>();
		
        public SkillConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SkillConfigCategory s = o as SkillConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SkillConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SkillConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SkillConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SkillConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SkillConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SkillConfig> GetList()
        {
            return this.list;
        }

        public SkillConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SkillConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>技能名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>施放类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>等级上限</summary>
		[ProtoMember(4)]
		public int LevelCap { get; set; }
		/// <summary>一级解锁等级</summary>
		[ProtoMember(5)]
		public int UnlockOne { get; set; }
		/// <summary>一级效果</summary>
		[ProtoMember(6)]
		public int EffectOne { get; set; }
		/// <summary>一级被动效果</summary>
		[ProtoMember(7)]
		public int PassiveEffectOne { get; set; }
		/// <summary>二级解锁等级</summary>
		[ProtoMember(8)]
		public int UnlockTwo { get; set; }
		/// <summary>二级效果</summary>
		[ProtoMember(9)]
		public int EffectTwo { get; set; }
		/// <summary>二级被动效果</summary>
		[ProtoMember(10)]
		public int PassiveEffectTwo { get; set; }
		/// <summary>三级解锁等级</summary>
		[ProtoMember(11)]
		public int UnlockThree { get; set; }
		/// <summary>三级效果</summary>
		[ProtoMember(12)]
		public int EffectThree { get; set; }
		/// <summary>三级被动效果</summary>
		[ProtoMember(13)]
		public int PassiveEffectThree { get; set; }
		/// <summary>四级解锁等级</summary>
		[ProtoMember(14)]
		public int UnlockFour { get; set; }
		/// <summary>四级效果</summary>
		[ProtoMember(15)]
		public int EffectFour { get; set; }
		/// <summary>四级被动效果</summary>
		[ProtoMember(16)]
		public int PassiveEffectFour { get; set; }
		/// <summary>五级解锁等级</summary>
		[ProtoMember(17)]
		public int UnlockFive { get; set; }
		/// <summary>五级效果</summary>
		[ProtoMember(18)]
		public int EffectFive { get; set; }
		/// <summary>五级被动效果</summary>
		[ProtoMember(19)]
		public int PassiveEffectFive { get; set; }
		/// <summary>六级解锁等级</summary>
		[ProtoMember(20)]
		public int UnlockSix { get; set; }
		/// <summary>六级效果</summary>
		[ProtoMember(21)]
		public int EffectSix { get; set; }
		/// <summary>六级被动效果</summary>
		[ProtoMember(22)]
		public int PassiveEffectSix { get; set; }
		/// <summary>七级解锁等级</summary>
		[ProtoMember(23)]
		public int UnlockSeven { get; set; }
		/// <summary>七级效果</summary>
		[ProtoMember(24)]
		public int EffectSeven { get; set; }
		/// <summary>七级被动效果</summary>
		[ProtoMember(25)]
		public int PassiveEffectSeven { get; set; }
		/// <summary>八级解锁等级</summary>
		[ProtoMember(26)]
		public int UnlockEight { get; set; }
		/// <summary>八级效果</summary>
		[ProtoMember(27)]
		public int EffectEight { get; set; }
		/// <summary>八级被动效果</summary>
		[ProtoMember(28)]
		public int PassiveEffectEight { get; set; }
		/// <summary>九级解锁等级</summary>
		[ProtoMember(29)]
		public int UnlockNine { get; set; }
		/// <summary>九级效果</summary>
		[ProtoMember(30)]
		public int EffectNine { get; set; }
		/// <summary>九级被动效果</summary>
		[ProtoMember(31)]
		public int PassiveEffectNine { get; set; }
		/// <summary>十级解锁等级</summary>
		[ProtoMember(32)]
		public int UnlockTen { get; set; }
		/// <summary>十级效果</summary>
		[ProtoMember(33)]
		public int EffectTen { get; set; }
		/// <summary>十级被动效果</summary>
		[ProtoMember(34)]
		public int PassiveEffectTen { get; set; }
		/// <summary>技能动作</summary>
		[ProtoMember(35)]
		public string Action { get; set; }
		/// <summary>技能图标</summary>
		[ProtoMember(36)]
		public string Icon { get; set; }
		/// <summary>技能描述</summary>
		[ProtoMember(37)]
		public string Describe { get; set; }

	}
}
