using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SkillEffectConfigCategory : ProtoObject, IMerge
    {
        public static SkillEffectConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SkillEffectConfig> dict = new Dictionary<int, SkillEffectConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SkillEffectConfig> list = new List<SkillEffectConfig>();
		
        public SkillEffectConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SkillEffectConfigCategory s = o as SkillEffectConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SkillEffectConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SkillEffectConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SkillEffectConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SkillEffectConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SkillEffectConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SkillEffectConfig> GetList()
        {
            return this.list;
        }

        public SkillEffectConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SkillEffectConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>效果类型</summary>
		[ProtoMember(2)]
		public int EffectConfig { get; set; }
		/// <summary>子弹图标</summary>
		[ProtoMember(3)]
		public string Bullet { get; set; }
		/// <summary>数值表达式</summary>
		[ProtoMember(4)]
		public string NumericExpression { get; set; }

	}
}
