using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class TextConfigCategory : ProtoObject, IMerge
    {
        public static TextConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, TextConfig> dict = new Dictionary<int, TextConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<TextConfig> list = new List<TextConfig>();
		
        public TextConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            TextConfigCategory s = o as TextConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (TextConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public TextConfig Get(int id)
        {
            this.dict.TryGetValue(id, out TextConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (TextConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, TextConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<TextConfig> GetList()
        {
            return this.list;
        }

        public TextConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class TextConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>文本内容</summary>
		[ProtoMember(2)]
		public string Text { get; set; }

	}
}
