using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class WorldParametersConfigCategory : ProtoObject, IMerge
    {
        public static WorldParametersConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, WorldParametersConfig> dict = new Dictionary<int, WorldParametersConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<WorldParametersConfig> list = new List<WorldParametersConfig>();
		
        public WorldParametersConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            WorldParametersConfigCategory s = o as WorldParametersConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (WorldParametersConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public WorldParametersConfig Get(int id)
        {
            this.dict.TryGetValue(id, out WorldParametersConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (WorldParametersConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, WorldParametersConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<WorldParametersConfig> GetList()
        {
            return this.list;
        }

        public WorldParametersConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class WorldParametersConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>参数名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>参数字段</summary>
		[ProtoMember(3)]
		public string Field { get; set; }
		/// <summary>参数值</summary>
		[ProtoMember(4)]
		public int[] Value { get; set; }

	}
}
